//The following C code is based Raspberry PI.
//Because the slave select pin of Raspberry SPI interface cannot be controlled by software independently.
//4 GPIOs in Bitbang mode are used to imitate the SPI timing.
//“JDI_LCD_bitmap.c” in the source file include C array converted from hex file generated in iCECube. A sample file is attached.
//User can call “cfg()” in the main procedure to finish the configuration.

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <ctype.h>		/* isspace() */
#include <libgen.h>		/* basename() */

#include "defs.h"		//Func prototypes
#include "bcm2835.h"


//extern char bitmap[];

#define EXTRAS 	0x08		//Send 8 dummy bytes (64 transitions) , after CDONE is high

#define SDO RPI_BPLUS_GPIO_J8_35

#define SCLK RPI_BPLUS_GPIO_J8_36

#define CSN RPI_BPLUS_GPIO_J8_37

#define SDI RPI_BPLUS_GPIO_J8_38

#define CRESETB RPI_BPLUS_GPIO_J8_40

#define CDONE RPI_BPLUS_GPIO_J8_15

#define PGM_VERSION     "1.02"
#define MIN_ARGS        1	// infile

#define PGM_VERSION     "1.02"



#define CR              0x0d
#define LF              0x0a

#define MAX_BUF_LEN    1024
#define MAX_LEN         255

#define MIN_ARGS        1	// infile


//****************************************************************************************************
// Global Var declarations
//****************************************************************************************************


extern char basefilename[MAX_BUF_LEN];
extern char buf1[MAX_BUF_LEN + 2];
extern char buf2[MAX_BUF_LEN + 2];
extern char buf3[MAX_BUF_LEN + 2];
extern char buf4[MAX_BUF_LEN + 2];


void
assert_sdo ()
{

  bcm2835_gpio_write (SDO, HIGH);

}

void
dessert_sdo ()
{

  bcm2835_gpio_write (SDO, LOW);

}

void
assert_sclk ()
{

  bcm2835_gpio_write (SCLK, HIGH);

}

void
dessert_sclk ()
{

  bcm2835_gpio_write (SCLK, LOW);

}

void
sendbyte (char data)
{

  char temp;

  int i;

  temp = data;

  for (i = 0; i < 8; i++)

    {

      if ((temp & 128) > 0)

	assert_sdo ();

      else

	dessert_sdo ();

      assert_sclk ();

      dessert_sclk ();

      temp = temp << 1;

    }

}

void
assert_ss ()
{

  bcm2835_gpio_write (CSN, LOW);

}

void
dessert_ss ()
{

  bcm2835_gpio_write (CSN, HIGH);

}

void
init_spi ()
{

  bcm2835_gpio_fsel (CSN, BCM2835_GPIO_FSEL_OUTP);

  bcm2835_gpio_fsel (SCLK, BCM2835_GPIO_FSEL_OUTP);

  bcm2835_gpio_fsel (CRESETB, BCM2835_GPIO_FSEL_OUTP);

  bcm2835_gpio_fsel (SDO, BCM2835_GPIO_FSEL_OUTP);

  bcm2835_gpio_fsel (SDI, BCM2835_GPIO_FSEL_INPT);

  bcm2835_gpio_fsel (CDONE, BCM2835_GPIO_FSEL_INPT);

  //bcm2835_gpio_write(CSN, HIGH);

  //bcm2835_gpio_write(SCLK, HIGH);

  //bcm2835_gpio_write(SDO, LOW);

}



int
cfg (FILE * infile)
{



  FILE *fp;
  int filesize = 0, progsize = 0, i = 0, j = 0, k = 0, size = 0;
  char *ptr;
  ssize_t bytes_read = 0;

  init_spi ();
  printf ("Program is running!\n");


  fp = infile;

  fseek (fp, 0L, SEEK_END);	//Seek to end , to get size
  filesize = ftell (fp);
  printf ("Filesize = %d\n", filesize);

  fseek (fp, 0L, SEEK_SET);	// Seek to beginning for reading.

  assert_ss ();

  char pdone = 0;


  bcm2835_gpio_write (CRESETB, LOW);

  bcm2835_delay (10);

  while (bcm2835_gpio_lev (CDONE) == HIGH)
    {
    };

  printf ("CDONE is low!\n");
  bcm2835_gpio_write (CRESETB, HIGH);

  delay (2000);
  //bcm2835_gpio_write(CRESETB, LOW);
  //delay(2000);

  bcm2835_delay (1);


  do
    {
      bytes_read = fread (buf1, 1, MAX_BUF_LEN, fp);

      if (bytes_read != 0)
	{
	  printf ("Bytes read = %d\n", bytes_read);
	  ptr = buf1;

	  for (i = 0; i < bytes_read; i++)
	    {
	      //printf ("running 1\n");

	      if (progsize <= filesize)
		{
		  sendbyte (*ptr++);
		  //printf ("running 2 i=%d\n",i);
		  progsize++;
		}
	      else
		{
		  j++;
		  //printf ("running =3 j=%d\n",j);

		  sendbyte (0);
		}

	      if ((pdone == 0) && (bcm2835_gpio_lev (CDONE) == HIGH))
		{
		  k = 0;
		  //
		  // Send minimum 49 spi clockcycles after CDONE has gone high
		  // See : TN1248, iCE40 Programming and Configuration , http://www.latticesemi.com/
		  // pg. 19
		  //
		  for (k = 0; k < EXTRAS; k++)
		    sendbyte (0);


		  printf ("CDONE is High! End of Configuration!\n");

		  pdone = 1;

		  break;

		}		// if((pdone == ....

	    }

	  printf ("Programmed %d bytes of %d total\n", progsize, filesize);
	}			// end if(byresread != ... 


    }
  while ((bytes_read != 0) || (pdone == 0));


  if (pdone == 0)

    {

      printf ("CDONE is Low! Configuration Error!");
      printf ("Programmed %d bytes of %d total\n", progsize, filesize);

    }

  dessert_ss ();
  return 0;
}
